/* competition preparation and survey is the pre-stage of whole development*/

# 2. 课题背景与调研

## 2.1 docker原理：容器与进程容器

    随着互联网业务架构由集群负载均衡转向微服务架构，众多微服务应用以容器化部署在云端平台，传统的虚拟化技术在微服务时代因其资源隔离的“厚重”深受诟病，而以docker为代表的应用容器引擎逐渐占领容器应用市场。docker等应用容器引擎利用Linux内核自带的NameSpace、Cgroups特性，为微服务业务带来了全新的轻量级虚拟化体验。容器是一种应用层抽象，用于将代码和依赖资源打包在一起。多个容器可以在同一台机器上运行，共享操作系统内核。这使得容器的隔离性相对较弱，带来安全上的风险，最严重时会导致容器逃逸，严重影响底层基础设施的保密性、完整性和可用性。我们将直接运行在裸机环境中的进程成为普通进程，**将运行在特定Namespace隔离环境（docker等容器引擎提供的隔离环境）中的裸机进程称为容器或者容器进程。**

#### Namespace介绍

Linux Namespace 是 Linux 提供的一种内核级别环境隔离的方法。这种隔离机制和 chroot 很类似，chroot 是把某个目录修改为根目录，从而无法访问外部的内容。Linux Namesapce 在此基础之上，提供了对 UTS、IPC、Mount、PID、Network、User 等的隔离机制，如下图所示。

| 分类               | 系统调用参数  | 相关内核版本                       |
| :----------------- | :------------ | :--------------------------------- |
| Mount Namespaces   | CLONE_NEWNS   | Linux 2.4.19                       |
| UTS Namespaces     | CLONE_NEWUTS  | Linux 2.6.19                       |
| IPC Namespaces     | CLONE_NEWIPC  | Linux 2.6.19                       |
| PID Namespaces     | CLONE_NEWPID  | Linux 2.6.19                       |
| Network Namespaces | CLONE_NEWNET  | 始于Linux 2.6.24完成于Linux 2.6.29 |
| User Namespaces    | CLONE_NEWUSER | 始于Linux 2.6.23完成于Linux 3.8    |

namespace 有三个系统调用可以使用：

* clone() --- 实现线程的系统调用，用来创建一个新的进程，并可以通过设计上述参数达到隔离。
* unshare() --- 使某个进程脱离某个 namespace
* setns(int fd, int nstype) --- 把某进程加入到某个 namespace

#### Cgroup介绍

CGroup 是 Control Groups 的缩写，是 Linux 内核提供的一种可以限制、记录、隔离进程组 (process groups) 所使用的物力资源 (如 cpu memory i/o 等等) 的机制。2007 年进入 Linux 2.6.24 内核，CGroups 不是全新创造的，它将进程管理从 cpuset 中剥离出来，作者是 Google 的 Paul Menage。CGroups 也是 LXC 为实现虚拟化所使用的资源管理手段。

#### CGroup 功能及组成

CGroup 是将任意进程进行分组化管理的 Linux 内核功能。CGroup 本身是提供将进程进行分组化管理的功能和接口的基础结构，I/O 或内存的分配控制等具体的资源管理功能是通过这个功能来实现的。这些具体的资源管理功能称为 CGroup 子系统或控制器。CGroup 子系统有控制内存的 Memory 控制器、控制进程调度的 CPU 控制器等。运行中的内核可以使用的 Cgroup 子系统由/proc/cgroup 来确认。

CGroup 提供了一个 CGroup 虚拟文件系统，作为进行分组管理和各子系统设置的用户接口。要使用 CGroup，必须挂载 CGroup 文件系统。这时通过挂载选项指定使用哪个子系统。

cgroups task_struct reference:

https://www.infoq.cn/article/docker-kernel-knowledge-cgroups-resource-isolation/

https://blog.csdn.net/punk_lover/article/details/78376430

> cgroup 指针指向了一个 cgroup 结构，也就是进程属于的 cgroup。进程受到子系统的控制，实际上是通过加入到特定的 cgroup 实现的，因为 cgroup 在特定的层级上，而子系统又是附和到上面的。通过以上三个结构，进程就可以和 cgroup 连接起来了：task_struct->css_set->cgroup_subsys_state->cgroup。


```c
static void fill_container_id(char *container_id) {
  struct task_struct *curr_task;
  struct css_set *css;
  struct cgroup_subsys_state *sbs;
  struct cgroup *cg;
  struct kernfs_node *knode, *pknode;
 
  curr_task = (struct task_struct *) bpf_get_current_task();
  css = curr_task->cgroups;
  bpf_probe_read(&sbs, sizeof(void *), &css->subsys[0]);
  bpf_probe_read(&cg,  sizeof(void *), &sbs->cgroup);
 
  bpf_probe_read(&knode, sizeof(void *), &cg->kn);
  bpf_probe_read(&pknode, sizeof(void *), &knode->parent);
 
  if(pknode != NULL) {
    char *aus;
 
    bpf_probe_read(&aus, sizeof(void *), &knode->name);
    bpf_probe_read_str(container_id, CONTAINER_ID_LEN, aus);
  }
}
```

## 2.2 ebpf技术

**eBPF** 是一个通用执行引擎，能够高效地安全地执行基于系统事件的特定代码，可基于此开发性能分析工具**、 **网络数据包过滤** 、 **系统调用过滤** ，系统观测和分析等诸多场景。eBPF可以由hook机制在系统调用被使用时触发，也可以通过kprobe或uprobe将eBPF程序附在内核/用户程序的任何地方。eBPF是将原先的BPF发展成一个指令集更复杂、应用范围更广的“内核虚拟机”。eBPF支持在用户态将C语言编写的一小段“内核代码”注入到内核中运行，注入时要先用llvm编译得到使用BPF指令集的elf文件，然后从elf文件中解析出可以注入内核的部分，最后用 `bpf_load_program`方法完成注入。 用户态程序和注入到内核中的程序通过共用一个位于内核中map实现通信。为了防止注入的代码导致内核崩溃，eBPF会对注入的代码进行严格检查，拒绝不合格的代码的注入。

通过ebpf技术，可以实现内核级别的数据过滤、跟踪以及监控。这些机制让eBPF的跟踪技术可以有效地感知容器的各项行为，包括但不限于：

* 容器对文件的访问
* 容器对系统的调用
* 容器之间的互访

# 2. 研究目标与研究方法

本项目由两部分组成：

* 一个零基础入门 `eBPF` 技术的教程实践和对应的命令行工具，使用C语言开发, 同时作为原型验证;
* 一个基于 `eBPF` 技术实现的用于监控容器的工具, 包含 `profile`、容器集群网络可视化分析、容器安全感知告警、一键部署、持久化存储监控等功能, 主要使用 Go 语言开发, 力求为工业界提供覆盖容器全生命周期的轻量级开源监控解决方案;

理论部分，目标旨在降低新手学习eBPF技术的门槛，试图通过大量的例程解释、丰富对eBPF、libbpf、bcc等内核技术的认知，该部分来自于学习实践过程中积累的各类学习资料、与开发者对eBPF技术逐步深化认知的过程。同时，结合本课题项目的来源，将实践部分拆分为X个实验Labs，配以详细丰富的踩坑经验。各Lab通过设置对操作系统不同主题的实验，进一步加深学生对操作系统中进程、线程、Tcp、文件系统等概念以及 `namespace` `、cgroup` 等内核机制的直观认知。

实践部分，来源于下述课题，主要是以C语言通过libbpf技术实现了一个用于监控容器行为的工具。开发过程符合软件工程开发规范，过程文档详细充实，测试用例丰富。每行代码均有相关注释。工具包含 `profile`、容器集群网络可视化分析、容器安全感知告警、一键部署、持久化存储监控等功能, 主要使用 Go 语言开发, 力求为工业界提供覆盖容器全生命周期的轻量级开源监控解决方案。

旨在：作为操作系统课程、 `eBPF` 开发入门、`docker`机制学习的补充材料,


# 3. 课题研发内容与初步计划

## 3.1 课题研发内容

1. 一份ebpf技术的入门教程与lab
2. 实现一个接近工业界的工具
3. 输出一系列开发文档（合订）

* 开发工具的使用教程——环境搭建；工具说明书；
* 开发工具的设计文档——设计架构、各模块详细设计、数据流程；
* 开发工具的开发文档——各模块的作用、函数的注释；
* 测试文档——测试环境的搭建教程；设计多种容器环境，进行测试；
* 过程文档——时间段、分工、迭代版次&版本变化；

## 3.2 课题研发模块

### 3.2.1 ebpf入门教程与Lab设计

1. ebpf介绍
2. 介绍bcc、libbpf与demo
3. 基于libbpf-bootstrap的系统跟踪、监控
4. Seccomp权限准入机制
5...

### 3.2.2 课题工具研发设计

**功能设计**

1. 进程信息监控
2. 系统调用跟踪
3. 进程间通信监控
4. Tcp流量端监控
5. 进程-容器映射
6. ...

**研发文档**

1. 调研文档
2. 系统设计文档
3. 模块设计文档
4. 开发计划文档
5. 版本更新文档
6. 开发日志文档
7. issue文档
8. 测试场景设计文档
9. 测试报告
10. 性能分析报告（可选）

## 3.3 研发计划

阶段一：学习ebpf相关技术栈（3.10~4.2）

* [ ] 入门ebpf技术栈
* [ ] 调研、学习 `bcc`
* [ ] 调研、学习 `libbpf` 、`libbpf-bootstrap`
* [ ] 调研、学习 `seccomp`
* [ ] 输出调研文档

阶段二：项目设计（4.3~4.10）

* [ ] 与mentor讨论项目需求、并设计功能模块
* [ ] 输出系统设计文档
* [ ] 输出模块设计文档

阶段三：开发迭代（4.10~6.1）

* [ ] 实现进程信息监控（pid、ppid等）
* [ ] 实现系统调用信息监控
* [ ] 实现进程间通信监控
* [ ] 实现tcp（ipv4、ipv6）通信监控
* [ ] 实现监控信息存储功能（csv或json格式）
* [ ] 完成了系统的原型验证功能
* [ ] 基于上述功能，实现命令行调用，完成版本v0.1
* [ ] 输出开发v0.1日志文档
* [ ] 实现进程id与容器id映射，进程信息过滤
* [ ] 添加“seccomp”功能
* [ ] 基于上述新增功能，迭代版本v0.2
* [ ] 输出开发v0.2日志文档
* [ ] 添加可视化模块
* [ ] 基于上述新增功能，迭代版本v0.3
* [ ] 输出开发v0.3日志文档
* [ ] 后续更新迭代

阶段四：开发测试（6.2~6.16）

* [ ] 设计测试场景（分别针对基础功能、权限控制、安全逃逸场景）
* [ ] 搭建测试环境
* [ ] 测试-开发
* [ ] 输出测试文档

### 2.3.1 vltrace

### 2.3.2 kindling

### 2.3.3 tracee

# 参考资料

1. 基于 eBPF 实现容器运行时安全

   https://mp.weixin.qq.com/s/UiR8rjTt2SgJo5zs8n5Sqg
2. 基于ebpf统计docker容器网络流量

   https://blog.csdn.net/qq_32740107/article/details/110224623
3. BumbleBee: Build, Ship, Run eBPF tools

   https://www.solo.io/blog/solo-announces-bumblebee/
4. Container traffic visibility library based on eBPF

   https://github.com/ntop/libebpfflow
5. about libbpf

   https://nakryiko.com/posts/libbpf-bootstrap/#why-libbpf-bootstrap
   https://nakryiko.com/posts/bpf-core-reference-guide/
6. bcc to libbpf

   https://nakryiko.com/posts/bcc-to-libbpf-howto-guide/#setting-up-user-space-parts
7. good intro for trace point and kprobe in ebpf

   https://www.iserica.com/posts/brief-intro-for-tracepoint/
   https://www.iserica.com/posts/brief-intro-for-kprobe/
8. other

   https://lockc-project.github.io/book/index.html
   https://github.com/willfindlay/bpfcontain-rs
9. user space uprobe

   https://www.collabora.com/news-and-blog/blog/2019/05/14/an-ebpf-overview-part-5-tracing-user-processes/
10. ebpf secomp

    https://developers.redhat.com/articles/2021/12/16/secure-your-kubernetes-deployments-ebpf#how_does_the_bpf_recorder_work_

    https://github.com/kubernetes-sigs/security-profiles-operator/blob/main/internal/pkg/daemon/bpfrecorder/bpf/recorder.bpf.c
11. tools

    https://github.com/iovisor/bcc/tree/master/libbpf-tools
